/* Sample C kernel to demonstrate MILEPOST optimization prediction */

#include <stdio.h>
#include	<stdlib.h>
#include	<string.h>

#define	MAX_BUFFS	20
static int library_init_done = FALSE;
static char *buf[MAX_BUFFS];		/* Memory for each open buf */
static long  buf_off[MAX_BUFFS];	/* File pointer for each buf */
static long  buf_size[MAX_BUFFS];	/* Count of bytes allocated for each buf */
static long  fds[MAX_BUFFS];		/* File descriptor status */
static int   buf_mode[MAX_BUFFS];

static void mem_init ()
{
	int i;

	for (i = 0; i < MAX_BUFFS; i++)
	{
		fds[i] = -1;
		buf[i] = (char *)NULL;
		buf_size[i] = 0;
		buf_off[i] = 0;
	}
}

int main(void *buffer, int size, char *mode)
{
	int ret, i;
	void *tmp;

	if (library_init_done == FALSE)
	{
		mem_init ();
		library_init_done = TRUE;
	}

	ret = -1;

	/* Find a free fd */

	for (i = 0; i < MAX_BUFFS; i++)
	{
		if (fds[i] == -1)
		{
			ret = i;
			break;
		}
	}

	if (i == MAX_BUFFS)		/* No more free descriptors */
	{
		ret = -1;
	}

	if (ret >= 0 && *mode == 'r')
	{
		if (buffer == (void *)NULL)
		{
			ret = -1;
		}
		else
		{
			buf[ret] = (char *)buffer;
			buf_size[ret] = size;
			buf_off[ret] = 0;
		}
	}
	else if (ret >= 0 && *mode == 'w')
	{

		if (buffer != (void *)NULL)
		{
			ret = -1;
		}

		else
		{
			tmp = malloc (0);	/* Get a pointer */
			if (tmp == (void *)NULL)
			{
				ret = -1;
			}
			else
			{
				buf[ret] = (char *)tmp;
				buf_size[ret] = 0;
				buf_off[ret] = 0;
			}
		}
	}
	else if (ret >= 0 && *mode == 'a')
	{
		if (buffer == (void *) NULL)	/* Create space for client */
		{
			tmp = malloc (0);	/* Get a pointer */
			if (tmp == (void *)NULL)
			{
				ret = -1;
			}
			else
			{
				buf[ret] = (char *)tmp;
				buf_size[ret] = 0;
				buf_off[ret] = 0;
			}
		}
		else							/* Client has file read in already */
		{
			buf[ret] = (char *)buffer;
			buf_size[ret] = size;
			buf_off[ret] = 0;
		}
	}
	else		/* Some other invalid combination of parameters */
	{
		ret = -1;
	}

	if (ret != -1)
	{
		fds[ret] = 0;
		buf_mode[ret] = *mode;
	}

	return (ret);
}